unit ZitateServerDB;

{$mode objfpc}{$H+}
{$I OsWahl.inc}
{$I ZitateParm.inc}

interface

uses
   Classes, SysUtils, ZitateDB, ZitateMsg;

{ TZitadell }

const
   daten_slash = {$IFDEF Linux} '/' {$ELSE} '\' {$ENDIF};

type
   TZitadellDB = class(TObject)
      constructor create;
      function sucheDatei(sDateiname: string): boolean;
      function sucheSQLiteDatabase(dbname: string): boolean;
      function openSQLiteDatabase: boolean;
      procedure zitadellPfad(psteuer: Integer);
      function addPfad(eins, zwei: string):string;
   private
      { $IFNDEF Linux}
      function GetHomeZitadell(Extension: string):string;
      {$IFNDEF Linux}
      function GetSpecialFolder(Folder: Integer): String;
      function GetRegZitadell:string;
      {$ENDIF}
      function GetUserDirZitadell(Extension: string):string;
      function GetEnvZitadell:string;
      function testSQLiteTabelle: boolean;
      function anlegenSQLiteTabelle: boolean;
   private
      zpfad, zdatabase, zdateiname:           string;
      zstill, zdblokal, zdateilokal, zonline: boolean;
   public
      property still: boolean read zstill write zstill;
      property online: boolean read zonline write zonline;
      property daten_pfad: string read zpfad write zpfad;
      property daten_database: string read zdatabase write zdatabase;
   protected
      property daten_dateiname: string read zdateiname write zdateiname;
      property daten_dateilokal: boolean read zdateilokal write zdateilokal;
      property daten_dblokal: boolean read zdblokal write zdblokal;
   end;

var
   ZitadellDB: TZitadellDB;

implementation

uses Dos {$IFNDEF Linux}, ShlObj, Registry {$ENDIF}, LConvEncoding;

{ Pfad... }

function TZitadellDB.addPfad(eins, zwei: string):string;
var temp: string;
begin
   temp:=eins;
   if (temp <> '') and
      (temp[length(temp)] <> daten_slash) and
      (zwei <> '')
      then temp:=temp+daten_slash;
   temp:=temp+zwei;
   Result:=temp;
end;

{$IFNDEF Linux}
function TZitadellDB.GetSpecialFolder(Folder: Integer): String;
begin
   SetLength(Result,MAX_PATH);
   If SHGetSpecialFolderPath(0, PChar(Result), Folder, false)
      then SetLength(Result,StrLen(PChar(Result)))
      else Result:='';
end;
{$ENDIF}

function TZitadellDB.GetHomeZitadell(Extension: string):string;
var tempdir, temppfad: string;
begin
   {$IFDEF Linux}
   tempdir:=getenv('HOME');
   {$ELSE}
   tempdir:=GetSpecialFolder(CSIDL_PERSONAL);
   {$ENDIF}
   tempdir:=addpfad( tempdir, Extension );
   temppfad:=addpfad( tempdir, daten_database );
   if DirectoryExists(tempdir) and FileExists(temppfad)
      then result:=tempdir else result:='';
end;

{$IFNDEF Linux}
function TZitadellDB.GetRegZitadell:string;
var Registry: TRegistry;
    tempdir, temppfad: string;
begin
   Registry := TRegistry.Create; tempdir:='';
   try
      Registry.RootKey := HKEY_LOCAL_MACHINE;
      if Registry.OpenKeyReadOnly('\SOFTWARE\Alfware\Zitadell')
         then tempdir:= Registry.ReadString('');
   finally
      Registry.Free;
   end;
   temppfad:=addpfad( tempdir, daten_database );
   if DirectoryExists(tempdir) and FileExists(temppfad)
      then result:=tempdir else result:='';
end;
{$ENDIF}

function TZitadellDB.GetUserDirZitadell(Extension: string):string;
var tempdir, temppfad:string;
begin
   tempdir:=addpfad(GetUserDir,Extension);
   temppfad:=addpfad( tempdir, daten_database );
   if DirectoryExists(tempdir) and FileExists(temppfad)
      then result:=tempdir else result:='';
end;

function TZitadellDB.GetEnvZitadell:string;
var tempdir, temppfad:string;
begin
   tempdir:=GetEnv('ZITADELL');
   temppfad:=addpfad( tempdir, daten_database );
   if DirectoryExists(tempdir) and FileExists(temppfad)
      then result:=tempdir else result:='';
end;

//0:  Registry  (* nur Windows *)
//1:  Environment-Variable Zitadell
//2:  Home (Eigene Dateien)
//3:  User
procedure TZitadellDB.zitadellPfad(psteuer: Integer);
begin
   if daten_pfad <> '' then exit;
   case psteuer of
   {$IFNDEF Linux}
   0: begin
         (* Registry *)
         if daten_pfad = '' then daten_pfad:= GetRegZitadell;
      end;
   {$ENDIF}
   2: begin
         (* Home-Dir 1.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetHomeZitadell('documents');
         (* Home-Dir 2.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetHomeZitadell(addPfad('documents','zitadell'));
         (* Home-Dir 3.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetHomeZitadell('Dokumente');
         (* Home-Dir 4.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetHomeZitadell(addPfad('Dokumente','zitadell'));
      end;
   { $ENDIF}
   1: begin
         (* Environment Variable ZITADELL *)
         if daten_pfad = '' then daten_pfad:= GetEnvZitadell;
      end;
   3: begin
         (* User-Dir 1.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetUserDirZitadell('');
         (* User-Dir 2.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetUserDirZitadell('zitadell');
         (* User-Dir 3.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetUserDirZitadell('documents');
         (* User-Dir 4.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetUserDirZitadell(addPfad('documents','zitadell'));
         (* User-Dir 5.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetUserDirZitadell('Dokumente');
         (* User-Dir 6.Versuch *)
         if daten_pfad = '' then daten_pfad:= GetUserDirZitadell(addPfad('Dokumente','zitadell'));
      end;
   else
   end;
end;

{ ZitateServer Constructor }

constructor TZitadellDB.Create;
begin
   daten_dateiname:= ''; daten_database:= 'dbTest.s3db'; daten_pfad:='';
   daten_dblokal:= false; daten_dateilokal:= false;
   still:= true; online:= false;
   {$IFNDEF ZITATE_DB_ANLEGEN_ERLAUBT} zitateDB1.anlegen_erlaubt:=false; {$ENDIF}
   //SQLite Connect erfolgt im zitateDB !
end;

function TZitadellDB.sucheDatei(sDateiname: string): boolean;
var vorhanden: boolean;
begin
   zitadellPfad(0);
   zitadellPfad(1);
   zitadellPfad(2);
   zitadellPfad(3);
   vorhanden:= false;
   if sDateiname <> '' then begin
      daten_dateiname:= sDateiname;
      if fileExists( daten_dateiname ) then begin
         daten_dateilokal:= true;
         if not still then SystemMeldung( 'Dateiname: ' + daten_dateiname + ' gefunden' );
         vorhanden:= true;
      end;
      if ( not vorhanden ) and fileExists( addPfad(daten_pfad, daten_dateiname) ) then begin
         daten_dateilokal:= false;
         if not still then SystemMeldung( 'Dateiname: ' + addPfad(daten_pfad, daten_dateiname)+ ' gefunden' );
         vorhanden:= true;
      end;
      Result:= vorhanden;
   end;
end;

function TZitadellDB.sucheSQLiteDatabase(dbname: string): boolean;
var vorhanden: boolean;
begin
   zitadellPfad(0);
   zitadellPfad(1);
   zitadellPfad(2);
   zitadellPfad(3);
   vorhanden:= false;
   if dbname <> '' then daten_database:= dbname;
   if fileExists( daten_database ) then begin
      daten_dblokal:= true;
      if not still then SystemMeldung( 'Datenbank: ' + daten_database + ' gefunden' );
      vorhanden:= true;
   end;
   if (not vorhanden) and fileExists( addPfad(daten_pfad, daten_database) ) then begin
      daten_dblokal:= false;
      if not still then SystemMeldung( 'Datenbank: ' + addPfad(daten_pfad, daten_database) + ' gefunden' );
      vorhanden:= true;
   end;

   if not vorhanden then begin
      if zitateDB1.anlegen_erlaubt then begin
         daten_dblokal:= true;
         vorhanden:= true;
         if not still then SystemMeldung( 'Datenbank: ' + daten_database + ' wird neu angelegt.' );
      end
      else
         if not still then SystemMeldung( 'Datenbank: ' + daten_database + ' darf nicht angelegt werden.' );
   end;
   Result:= vorhanden;
end;

function TZitadellDB.openSQLiteDatabase: boolean;
begin
    if not online then begin
       if daten_dblokal
          then online:= zitateDB1.dbConnectOK( daten_database )
          else online:= zitateDB1.dbConnectOK( addPfad(daten_pfad, daten_database) );

          if online then
             if not testSQLiteTabelle
	     then online:= ( anlegenSQLiteTabelle and testSQLiteTabelle );
    end;
    Result:= online;
end;

function TZitadellDB.testSQLiteTabelle: boolean;
begin
   Result:= zitateDB1.testTabellenOK;
end;

function TZitadellDB.anlegenSQLiteTabelle: boolean;
var dbOK:boolean;
begin
   dbOK:= zitateDB1.tabellenAnlegenOK;
   if dbOK then
      if not still then SystemMeldung( 'Die Tabellen in der Datenbank wurden neu angelegt' );
   Result:= dbOK;
end;

begin
   Randomize;
   ZitadellDB:=TZitadellDB.Create;
end.
